<?php

namespace App\Service\Action\Classes;

use App\Entity\User;
use App\Service\Action\BaseActionService;
use App\Service\Dto\Classes\RegisterCodeDto;
use App\Service\Dto\DtoServiceInterface;
use App\Service\Response\ResponseServiceInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\SecurityBundle\Security;

class CheckRegisterCode extends BaseActionService
{
    private ?User $user;

    /**
     * @param RegisterCodeDto $registerCodeDto
     * @param ResponseServiceInterface $response
     * @param ManagerRegistry $doctrine
     * @param Security $security
     */
    public function __construct(
        private DtoServiceInterface      $registerCodeDto,
        private ResponseServiceInterface $response,
        private ManagerRegistry          $doctrine,
        Security                         $security,
    )
    {
        $this->user = $security->getUser();
        parent::__construct($response);
    }

    /**
     * Подтверждение регистрации по коду
     *
     * @return void
     */
    public function runAction(): void
    {
        $currentDate = new \DateTime();
        $code = $this->registerCodeDto->getClass()->code;
        $registerCode = $this->user->getRegisterCode();
        if ($registerCode === null) {
            $this->response->addError('Код подтверждения не отправлен');
        } else {
            if ($registerCodeDate = $registerCode->getDate()) {
                if ($registerCode->getCode() === $code && $currentDate->getTimestamp() < $registerCodeDate->getTimestamp()) {
                    try {
                        $this->user->setConfirm(true);
                        $em = $this->doctrine->getManager();
                        $em->persist($this->user);
                        $em->remove($registerCode);
                        $em->flush();
                        $this->response->addMessage('Регистрация подтверждена');
                    } catch (\Exception $exception) {
                        $this->response->addError('Ошибка подтверждения регистрации');
                    }
                } else {
                    $this->response->addError('Код недействителен');
                }
            } else {
                $this->response->addError('Код недействителен');
            }
        }
    }

    public function validate(): bool
    {
        if ($this->user === null) {
            $this->response->addError('Вы не авторизованы');
            return false;
        }
        return $this->registerCodeDto->validate($this->response);
    }
}