<?php

namespace App\Service\Action\Classes;

use App\Entity\User;
use App\Service\Action\BaseActionService;
use App\Service\Response\Classes\ProfileResponse;
use App\Service\Response\ResponseServiceInterface;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\Security\Http\Attribute\CurrentUser;
use Symfony\Component\Serializer\SerializerInterface;

class GetProfile extends BaseActionService
{
    private ?User $user;

    /**
     * @param ProfileResponse $profileResponse
     * @param Security $security
     */
    public function __construct(
        private ResponseServiceInterface $profileResponse,
        Security                         $security
    )
    {
        $this->user = $security->getUser();
        parent::__construct($profileResponse);
    }


    /**
     * Получение профиля пользователя
     *
     * @return void
     *
     * @throws \JsonException
     */
    public function runAction(): void
    {
        $this->profileResponse->setData($this->user);
    }

    public function validate(): bool
    {
        if ($this->user === null) {
            $this->profileResponse->addError('Вы не авторизованы');
            return false;
        }
        if ($this->user->isDeleted()) {
            $this->profileResponse->addError('Профиль удален');
            return false;
        }
        return true;
    }
}