<?php

namespace App\Service\Action\Classes;

use App\Entity\User;
use App\Service\Action\BaseActionService;
use App\Service\Dto\Classes\ChangePasswordDto;
use App\Service\Dto\DtoServiceInterface;
use App\Service\Response\ResponseServiceInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;

class ResetPassword extends BaseActionService
{
    private ?User $user;

    public function __construct(
        private ResponseServiceInterface    $response,
        private UserPasswordHasherInterface $passwordHasher,
        private DtoServiceInterface         $passwordDto,
        private ManagerRegistry             $doctrine,
        Security                         $security
    )
    {
        $this->user = $security->getUser();
        parent::__construct($response);
    }

    public function runAction(): void
    {
        /** @var ChangePasswordDto $dto */
        $dto = $this->passwordDto->getClass();

        if ($this->passwordHasher->isPasswordValid($this->user, $dto->oldPassword)) {
            $hashedPassword = $this->passwordHasher->hashPassword(
                $this->user,
                $dto->password ?: ''
            );
            $this->user->setPassword($hashedPassword);

            try {
                $em = $this->doctrine->getManager();
                $em->persist($this->user);
                $em->flush();
                $this->response->addMessage('Пароль изменен');
            } catch (\Exception $exception) {
                $this->response->addError('Ошибка изменения пароля');
            }
        } else {
            $this->response->addError('Текущий пароль неверен');
        }
    }

    public function validate(): bool
    {
        if ($this->user === null) {
            $this->response->addError('Вы не авторизованы');
            return false;
        }
        return $this->passwordDto->validate($this->response);
    }
}